//
//  LibraryViewController.swift
//  qPadLab
//
//  Created by David Arcos Gutierrez on 25/5/18.
//  Copyright © 2018 David Arcos Gutierrez. All rights reserved.
//

import UIKit

class LibraryViewController: UIViewController {

    @IBOutlet weak var navBarView: UIView?
    @IBOutlet weak var cellContainerView: UIView?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do initial interface setup and general style improvements
        interfaceSetup()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    /* ------------------------------------------------- */
    /*                BUTTON ACTIONS                     */
    /* ------------------------------------------------- */
    
    @IBAction func closeButton (sender: UIButton) {
        self.dismiss(animated:true, completion:nil)
    }
    
    @IBAction func sendLoadCircuit (sender: UIButton) {
        
        let notif = Notification.Name.init(rawValue: "loadCircuit")
        let cell = DataManager.shared.circuits[sender.tag]
        NotificationCenter.default.post(name: notif, object: cell)
        self.dismiss(animated:true, completion:nil)
    }
    
    /* ------------------------------------------------- */
    /*                  INTERFACE                        */
    /* ------------------------------------------------- */
    func interfaceSetup (){
        
        if let v = navBarView {
            StyleManager.shared.setBottomShadowToView(view: v)
        }
        
        if let v = cellContainerView {
            StyleManager.shared.setBottomShadowToView(view: v)
            let cellList = DataManager.shared.circuits
            
            let columns = 6
            let padding = 10.0
            
            let boxSize = (Double(v.frame.width) - Double(columns+1)*padding)/Double(columns)
            
            for (index, cell) in cellList.enumerated() {
                
                var myFrame = CGRect(x: padding + Double(index%columns)*(padding + boxSize),
                                   y: padding + Double(index/columns)*(padding + boxSize),
                                   width: boxSize,
                                   height: boxSize)
                
                let cellView = UIView.init(frame: myFrame)
                let isCurrentCell = DataManager.shared.currentCell.identifier == cell.identifier
                
                if isCurrentCell{
                    cellView.backgroundColor = UIColor.gray
                }else{
                    cellView.backgroundColor = UIColor.lightGray
                }
                
                //Name
                myFrame = CGRect(x: padding,
                                 y: padding,
                                 width: boxSize-2*padding,
                                 height: 20)
                let cellNameLabel = UILabel.init(frame: myFrame)
                cellNameLabel.text = cell.name
                
                //Number of gates
                myFrame = CGRect(x: padding,
                                 y: 2*padding + 20,
                                 width: boxSize-2*padding,
                                 height: 20)
                let cellgatesNumberLabel = UILabel.init(frame: myFrame)
                cellgatesNumberLabel.text = "\(cell.gates.count) gates"
                
                //Button
                myFrame = CGRect(x: padding,
                                 y: 3*padding + 40,
                                 width: boxSize-2*padding,
                                 height: 20)
                let loadCellButton = UIButton.init(frame: myFrame)
                loadCellButton.addTarget(self, action: #selector(self.sendLoadCircuit), for: .touchUpInside)
                loadCellButton.backgroundColor = UIColor.white
                loadCellButton.titleLabel?.text = "load"
                loadCellButton.tag = index
                
                cellView.addSubview(cellNameLabel)
                cellView.addSubview(cellgatesNumberLabel)
                cellView.addSubview(loadCellButton)
                
                v.addSubview(cellView)
                
            }
        }
    }
}
