//
//  CircuitEvaluator.swift
//  qPadLab
//
//  Created by David Arcos Gutierrez on 21/4/18.
//  Copyright © 2018 David Arcos Gutierrez. All rights reserved.
//

import UIKit

class CircuitEvaluator: NSObject {
    
    // Figure of merit: GARBAGE
    class func evaluateFigureGarbage(cell:QCell)->Int{
        
        //It requires user interpretation (output qubits)
        return 0
    }
    
    // Figure of merit: QUANTUM COST
    class func evaluateFigureQuantumCost(cell:QCell)->Int{
        
        // Without an optimization process it is equal to the sum of gate costs
        var qCost = 0
        for gate in cell.gates{
            qCost += gate.figures.cost
        }
        return qCost
    }
    
    // Figure of merit: REGISTER SIZE (it also codifies the name of the register's qubits)
    class func evaluateFigureRegisterSize(cell:QCell)->Int{
        
        // Every register line needs a qubit. It is equal to the sum of lines.
        var numberOfQubits:Int = 0
        var registerList:[Bool] = Array(repeating: false, count: StyleManager.shared.maxVerticalPoints + 1)
        for gate in cell.gates{
            if gate.type.rawValue == QBlockType.feynman.rawValue {
                registerList[gate.gridOrigin.y + 1] = true
                registerList[gate.gridOrigin.y + gate.gridRect.height - 1] = true
            }else{
                registerList[gate.gridOrigin.y + 2] = true
            }
        }
        
        cell.registerCodification = [Int: Int]()
        
        // Assign the name of register's qubits
        //  Top -> qubit 0
        //  Bottom -> qubit N-1 (N = register size)
        for (index, c) in registerList.enumerated(){
            if c == true {
                cell.registerCodification[index] = numberOfQubits
                numberOfQubits += 1
            }
        }
        
        return numberOfQubits
    }
    
    // Figure of merit: DELAY
    class func evaluateFigureDelay(cell:QCell)->Int{
        
        // Approximation: the number of columns of the cell
        // TODO: improve this algorithm
        var numberOfStages:Int = 0
        var evolutionStages:[Bool] = Array(repeating: false, count: StyleManager.shared.horitzontalPoints + 1)
        for gate in cell.gates{
            evolutionStages[gate.gridOrigin.x + 2] = true
        }
        
        for stage in evolutionStages{
            if stage == true {
                numberOfStages += 1
            }
        }
        
        return numberOfStages
    }
}
