//
//  DataManager.swift
//  qPadLab
//
//  Created by David Arcos Gutierrez on 23/5/18.
//  Copyright © 2018 David Arcos Gutierrez. All rights reserved.
//

import UIKit

class DataManager: NSObject {

    static let shared = DataManager()
    
    // List of user's circuits
    var circuits:[QCell] = Array()
    
    // Current circuit (a circuit copy shown in the grid)
    var currentCell: QCell = QCell(cellName: "quantum cell")
    
    // Document directory path
    var filePath: String {
        let url = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first
        return (url!.appendingPathComponent("Data").path)
    }
    
    // Add a circuit to the list
    func addQCell(cell:QCell) {
        
        // Update cell's management data
        cell.unregisteredCell = false
        cell.unsavedChanges = false
        
        circuits.append(cell.copy() as! QCell)
    }
    
    // Update current cell changes
    func updateCurrentCellChanges() {
        
        if currentCell.unregisteredCell {
            self.addQCell(cell: currentCell)
        }else{
            // Search a cell with the same identifier
            for (index, cell) in circuits.enumerated() {
                if currentCell.identifier == cell.identifier {
                    currentCell.unsavedChanges = false
                    // Override data by using current cell copy
                    circuits[index] = currentCell.copy() as! QCell
                }
            }
        }
        
        // Save data
        saveCircuits()
    }
    
    // Save circuit list data to memory
    func saveCircuits() {
        
        NSKeyedArchiver.archiveRootObject(self.circuits, toFile: filePath)
    }
    
    // Load circuit list data from memory
    func loadCirctuits() {

        if let ourData = NSKeyedUnarchiver.unarchiveObject(withFile: filePath) as? [QCell] {
            self.circuits = ourData
        }
    }
    
    // Save current circuit data to memory
    func saveCurrentCircuit() {
        
        NSLog("guardo el current \(currentCell.gates.count)")
        let defaults = UserDefaults.standard
        let encodedData: Data = NSKeyedArchiver.archivedData(withRootObject: currentCell)
        defaults.set(encodedData, forKey: "qPadLab_qCell_currentCell")
        defaults.synchronize()
    }
    
    // Load current circuit data from memory
    func loadCurrentCircuit() {
        
        NSLog("carrego el current")
        let defaults = UserDefaults.standard
        if let decoded = defaults.object(forKey: "qPadLab_qCell_currentCell") {
            currentCell = NSKeyedUnarchiver.unarchiveObject(with: decoded as! Data) as! QCell
        }
    }
    
    // Save tutorial shown flag to user's defaults
    func setTutorialShown() {
        
        let defaults = UserDefaults.standard
        defaults.set(true, forKey: "qPadLab_bool_tutorialShown")
        defaults.synchronize()
    }
    
    // Get tutorial shown flag from user's defaults
    func isTutorialShown() -> Bool {
        
        let defaults = UserDefaults.standard
        let notFirstTime = defaults.bool(forKey: "qPadLab_bool_tutorialShown")
        
        return notFirstTime
    }
}
