//
//  QBlockView.swift
//  qPadLab
//
//  Created by David Arcos Gutierrez on 18/4/18.
//  Copyright © 2018 David Arcos Gutierrez. All rights reserved.
//

import UIKit

struct gridReference {
    
    var x = 0
    var y = 0
}

struct gridRectPoints {
    
    var width = 0
    var height = 0
}

struct figuresOfMerit {
    
    //Default values
    var delay = 1
    var cost = 1
    var garbage = 0
}

public enum QBlockState {
    case undefined
    case gatebox
    case fixed
    case waiting
    case editable
    case deleted
}

public enum QBlockEditableSubstate {
    case wellPlaced
    case wrongPlaced
    case garbagePlaced
}

public enum QBlockType: String  {
    case undefined = "-"
    case hadamard = "H"
    case pauliX = "X"
    case pauliY = "Y"
    case pauliZ = "Z"
    case feynman = "CNot"
}

public class QBlockView: UIView, NSCopying {

    // Quantum block data
    var gridOrigin = gridReference()
    var gridRect = gridRectPoints()
    var figures = figuresOfMerit()
    var state = QBlockState.undefined
    var type = QBlockType.undefined
    
    override init(frame: CGRect)
    {
        super.init(frame: frame)
    }
    
    convenience init(type: QBlockType)
    {
        self.init(frame: CGRect.zero)
        self.type = type
    }
    
    public func encodeWithCoder(aCoder: NSCoder) {
        
        super.encode(with: aCoder)
        
        aCoder.encode(gridOrigin.x, forKey: "gridOrigin_x")
        aCoder.encode(gridOrigin.y, forKey: "gridOrigin_y")
        aCoder.encode(gridRect.width, forKey: "gridRect_width")
        aCoder.encode(gridRect.height, forKey: "gridRect_height")
    }
    
    required public init?(coder aDecoder: NSCoder)
    {
        super.init(coder: aDecoder)
        
        gridOrigin.x = aDecoder.decodeInteger(forKey: "gridOrigin_x")
        gridOrigin.y = aDecoder.decodeInteger(forKey: "gridOrigin_y")
        gridRect.width = aDecoder.decodeInteger(forKey: "gridRect_width")
        gridRect.height = aDecoder.decodeInteger(forKey: "gridRect_height")
        state = .fixed
    }
    
    public func copy(with zone: NSZone? = nil) -> Any {
        
        let copy = QBlockView.init(frame: self.frame)
        
        copy.gridRect = self.gridRect
        copy.gridOrigin = self.gridOrigin
        copy.figures = self.figures
        copy.state = self.state
        copy.type = self.type
        
        return copy
    }
}
