//
//  FeynmanGate.swift
//  qPadLab
//
//  Created by David Arcos Gutierrez on 24/4/18.
//  Copyright © 2018 David Arcos Gutierrez. All rights reserved.
//

import UIKit

class FeynmanGate: QBlockView {
    
    // Sketch parameters
    public var reversed: Bool = false // Default control up and target down
    public var target: FeynmanTarget = FeynmanTarget()
    
    override init(frame: CGRect)
    {
        super.init(frame: frame)
        self.type = .feynman
        self.backgroundColor = UIColor.init(red: 1.0,
                                            green: 1.0,
                                            blue: 1.0,
                                            alpha: 0.0)
    }
    
    convenience init(frame: CGRect, inputs:Int, reversed:Bool)
    {
        self.init(frame: frame)
        self.reversed = reversed
        
        self.gridRect.width = 4
        self.gridRect.height = inputs + 1
        
        //Figures of merit initialization
        figures.delay = 1
        figures.cost = 1
        figures.garbage = 0
    }
    
    public override func encode(with aCoder: NSCoder) {
        
        super.encodeWithCoder(aCoder: aCoder)
        
        aCoder.encode(reversed, forKey: "reversed")
    }
    
    required public init?(coder aDecoder: NSCoder) {
        
        super.init(coder: aDecoder)
        reversed = aDecoder.decodeBool(forKey: "reversed")
        type = .feynman
    }
    
    public override func copy(with zone: NSZone? = nil) -> Any {
        
        let copy = FeynmanGate.init(frame: self.frame)
        
        copy.gridRect = self.gridRect
        copy.gridOrigin = self.gridOrigin
        copy.figures = self.figures
        copy.state = self.state
        copy.type = self.type
        
        copy.reversed = self.reversed
        
        return copy
    }
    
    override func draw(_ rect: CGRect)
    {
        let padding = Double(rect.size.height)/(2.0*Double(self.gridRect.height))
        let width = Double(rect.size.width) - 2.0*padding
        let height = Double(rect.size.height) - 2.0*padding
        let circleRadius = 10.0
        let circleDiametre = 2.0*circleRadius
        let controlDotDiamtre = 6.0
        
        var aPath = UIBezierPath()
        
        aPath.lineWidth = 2.0
        UIColor.black.set()
        
        // Draw vertical line
        if reversed {
            aPath.move(to: CGPoint(x:padding + width/2.0, y:2.0*padding - circleRadius))
            aPath.addLine(to: CGPoint(x:padding + width/2.0, y:height))
        }else{
            aPath.move(to: CGPoint(x:padding + width/2.0, y:2.0*padding))
            aPath.addLine(to: CGPoint(x:padding + width/2.0, y:height + circleRadius))
        }
        aPath.stroke()
        
        // Draw register lines
        aPath.move(to: CGPoint(x:0, y:2*padding))
        aPath.addLine(to: CGPoint(x:width + 2.0*padding, y:2*padding))
        aPath.stroke()
        
        aPath.move(to: CGPoint(x:0, y:height))
        aPath.addLine(to: CGPoint(x:width + 2.0*padding, y:height))
        aPath.stroke()
        
        /* only target
         aPath.move(to: CGPoint(x:padding + width/2.0 - circleRadius, y:height))
         aPath.addLine(to: CGPoint(x:padding + width/2.0 + circleRadius, y:height))
         aPath.stroke()*/
        
        // Draw target
        var circleFrame: CGRect = CGRect.zero
        var circleInteractionFrame: CGRect = CGRect.zero
        
        if reversed {
            circleFrame = CGRect(x: padding + width/2.0 - circleRadius,
                                 y: 2*padding - circleRadius,
                                 width: circleDiametre,
                                 height: circleDiametre)
           
            circleInteractionFrame = CGRect(x: width/2.0 - padding,
                                            y: 0,
                                            width: padding*4.0,
                                            height: padding*4.0)
        }else{
            circleFrame = CGRect(x: padding + width/2.0 - circleRadius,
                                 y: height - circleRadius,
                                 width: circleDiametre,
                                 height: circleDiametre)
            
            circleInteractionFrame = CGRect(x: width/2.0 - padding,
                                            y: height - 2*padding,
                                            width: padding*4.0,
                                            height: padding*4.0)
        }
        
        aPath = UIBezierPath(ovalIn: circleFrame)
        aPath.lineWidth = 2.0
        aPath.stroke()
        
        // Include Feynman target subview if needed (when state is different from gatebox)
        if state.hashValue != QBlockState.gatebox.hashValue {
            target.frame = circleInteractionFrame
            target.backgroundColor = UIColor.yellow
            target.alpha = 0.2
            target.layer.cornerRadius = CGFloat(padding*2.0)
            self.addSubview(target)
        }
        
        // Draw control
        if reversed {
            circleFrame = CGRect(x: padding + width/2.0 - controlDotDiamtre/2.0,
                                 y: height - controlDotDiamtre/2.0,
                                 width: controlDotDiamtre,
                                 height: controlDotDiamtre)
        }else{
            circleFrame = CGRect(x: padding + width/2.0 - controlDotDiamtre/2.0,
                                 y: 2*padding - controlDotDiamtre/2.0,
                                 width: controlDotDiamtre,
                                 height: controlDotDiamtre)
        }
        
        aPath = UIBezierPath(ovalIn: circleFrame)
        aPath.fill()
        aPath.stroke()
    }
    
}
