//
//  PauliGate.swift
//  qPadLab
//
//  Created by David Arcos Gutierrez on 18/4/18.
//  Copyright © 2018 David Arcos Gutierrez. All rights reserved.
//

import UIKit

enum PauliType: String {
    case x = "X"
    case y = "Y"
    case z = "Z"
}

class PauliGate: QBlockView {
    
    override init(frame: CGRect)
    {
        super.init(frame: frame)
        self.backgroundColor = UIColor.init(red: 1.0,
                                            green: 1.0,
                                            blue: 1.0,
                                            alpha: 0.0)
        
        self.gridRect.width = 4
        self.gridRect.height = 4
    }
    
    convenience init(frame: CGRect, type:QBlockType)
    {
        self.init(frame: frame)
        self.type = type
        
        //Figures of merit initialization
        figures.delay = 1
        figures.cost = 1
        figures.garbage = 0
    }
    
    
    public override func encode(with aCoder: NSCoder) {
        
        super.encodeWithCoder(aCoder: aCoder)
        switch(type) {
        case .pauliX:
            aCoder.encode("X", forKey: "pauliType")
        case .pauliY:
            aCoder.encode("Y", forKey: "pauliType")
        default:
            aCoder.encode("Z", forKey: "pauliType")
        }
    }
    
    required public init?(coder aDecoder: NSCoder) {
        
        super.init(coder: aDecoder)
        
        let typeCode:String = aDecoder.decodeObject(forKey: "pauliType") as! String
        switch(typeCode) {
        case "X":
            type = .pauliX
        case "Y":
            type = .pauliY
        default:
            type = .pauliZ
        }
        
    }
    
    public override func copy(with zone: NSZone? = nil) -> Any {
        
        let copy = PauliGate.init(frame: self.frame)
        
        copy.gridRect = self.gridRect
        copy.gridOrigin = self.gridOrigin
        copy.figures = self.figures
        copy.state = self.state
        copy.type = self.type
        
        return copy
    }
    
    override func draw(_ rect: CGRect)
    {
        let padding = 2.0*Double(rect.size.height)/(2.0*Double(self.gridRect.height))
        let width = Double(rect.size.width) - 2.0*padding
        let height = Double(rect.size.height) - 2.0*padding
        
        let aPath = UIBezierPath()
        
        aPath.lineWidth = 2.0
        UIColor.black.set()
        
        // Draw box
        aPath.move(to: CGPoint(x:padding, y:padding))
        aPath.addLine(to: CGPoint(x:padding + width, y:padding))
        aPath.addLine(to: CGPoint(x:padding + width, y:padding + height))
        aPath.addLine(to: CGPoint(x:padding, y:padding + height))
        aPath.close()
        aPath.stroke()
        
        // Draw register lines
        aPath.move(to: CGPoint(x:0, y:padding + width/2.0))
        aPath.addLine(to: CGPoint(x:padding, y:padding + width/2.0))
        aPath.stroke()
        
        aPath.move(to: CGPoint(x:width + padding, y:padding + width/2.0))
        aPath.addLine(to: CGPoint(x:width + 2.0*padding, y:padding + width/2.0))
        aPath.stroke()
        
        // Add pauli type label
        let cellTitle = UILabel()
        cellTitle.frame = CGRect(x: padding,
                                 y: padding,
                                 width: width,
                                 height: height)
        
        cellTitle.textAlignment = NSTextAlignment.center
        cellTitle.text = self.type.rawValue
        cellTitle.font = cellTitle.font.withSize(CGFloat(1.6*padding))
        self.addSubview(cellTitle)
    }
}

